/*
 * Copyright 2004-2005, the OpenGLEAN Project.
 *
 * Portions copyright (C) 2004, the OpenGLUT project contributors.
 * OpenGLUT branched from freeglut in February, 2004.
 *
 * Copyright (c) 1999-2000 Pawel W. Olszta. All Rights Reserved.
 * Written by Pawel W. Olszta, <olszta@sourceforge.net>
 * Creation date: Thu Dec 2 1999
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * openglean_ext.h
 *
 * The non-GLUT-compatible extensions to the OpenGLEAN library include file
 *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */

#ifndef __OPENGLEAN_H__
#define __OPENGLEAN_H__


#ifdef __cplusplus
extern "C" {
#endif

/*
 * Under windows, we have to differentiate between static and dynamic libraries
 */
#if !defined (_MSC_VER) && !defined (__MINGW32__)
#    define OGAPI
#    define OGAPIENTRY
#else
#   define WIN32_LEAN_AND_MEAN
#   define NO_MIN_MAX
#   include <windows.h>
#   undef min
#   undef max
#   ifdef OPENGLEAN_STATIC         /* Windows static library */
#       define OGAPI
#       define OGAPIENTRY
#       if defined (_MSC_VER)      /* Link with Win32 static OpenGLEAN lib */
#           pragma comment (lib, "OpenGLEAN_static.lib")
#       endif
#   else                          /* Windows shared library (DLL) */
#       if defined (OPENGLEAN_EXPORTS)
#           define OGAPI __declspec (dllexport)
#       else
#           define OGAPI __declspec(dllimport)
#           if defined(_MSC_VER)  /* link with Win32 shared OpenGLEAN lib */
#               pragma comment (lib, "OpenGLEAN.lib")
#           endif
#       endif
#       define OGAPIENTRY __stdcall
#   endif
    /*
     * Although #pragma is supported by all ANSI C compilers,
     * it is apparently evaluated at the C preprocessor level,
     * hence compilers can---and some will---peek inside
     * of #ifdef-protected code, and may mis-interpretet
     * these #pragmas.
     *
     * Discussion by freeglut developers suggests that
     * #pragmas may need to move to a separate header.
     * See the archives leading up to 24th Dec 2003
     */
#   if defined (_WIN32_WCE)
#       pragma comment ( lib, "Aygshell.lib" )
#   endif
#   if defined(_MSC_VER)
        /* Drag in other Windows libraries as required by OpenGLEAN */
#       pragma comment (lib, "advapi32.lib") /* Joystick library?           */
#       pragma comment (lib, "winmm.lib")    /* link Windows MultiMedia lib */
#       pragma comment (lib, "user32.lib")   /* link Windows user lib       */
#       pragma comment (lib, "gdi32.lib")    /* link Windows GDI lib        */
#       pragma comment (lib, "opengl32.lib") /* link Microsoft OpenGL lib   */
#       pragma comment (lib, "glu32.lib")    /* link OpenGL Utility lib     */
        /* These warnings are never helpful */
#       pragma warning (disable:4244)
#       pragma warning (disable:4305)
#   endif
#endif

/*
 * Some helpful symbols for conditional compilation.
 */
#define OPENGLEAN 1
#define OPENGLEAN_VERSION_0_6_0

/*
 * Always include OpenGL and GLU headers
 */
#include <GL/gl.h>
#include <GL/glu.h>


/*
 * GLUT API macro definitions -- the special key codes:
 *
 * The *_HEAD and *_TAIL definitions are strictly for internal
 * purposes.  Data that you give to the library should always fall
 * into the indicated ranges---this is automatic if you use the
 * symbolic names.  Data that you get back from the library may or
 * may not fall into those ranges.
 *
 * Most OpenGLEAN symbolic constants are entered here.  The exceptions are:
 *
 *  * Fonts.  Fonts are currently {void *}.
 *
 *  * Bitmasks.  These can't use enum's automated numbering and
 *    generally restart at 1.  (But we could consider combining
 *    some unrelated bitmasks to reduce the overlap of bit
 *    definitions for unrelated functions.)  Also, the biggest
 *    problem with a bitmask is that you often see multiple bits
 *    ORed together, such that the debugger can't generally separate
 *    them.  Perhaps C needs something like {enum} for naming bitfields,
 *    while retaining the ability to do bitwise ops?  (A union of
 *    an integer with a bitfielded-struct is not quite the same
 *    thing.)
 *
 * We could still put everything but the pointers into enums,
 * though...
 *
 * XXX Consider breaking this big enum down into separate enums, by
 * XXX section.  Then we could assign a type to the enums and enlist
 * XXX compilers and other tools to check the usage of the symbols.
 */
enum
{
    /*
     * XXX We start with 1000, rather than 0, because there are
     * XXX boolean, and pseudo-boolean constants later that really
     * XXX want to occupy the low-lying numbers.
     */
    GLUT_KEY_HEAD = 1000,
        GLUT_KEY_F1,
        GLUT_KEY_F2,
        GLUT_KEY_F3,
        GLUT_KEY_F4,
        GLUT_KEY_F5,
        GLUT_KEY_F6,
        GLUT_KEY_F7,
        GLUT_KEY_F8,
        GLUT_KEY_F9,
        GLUT_KEY_F10,
        GLUT_KEY_F11,
        GLUT_KEY_F12,
        GLUT_KEY_LEFT,
        GLUT_KEY_UP,
        GLUT_KEY_RIGHT,
        GLUT_KEY_DOWN,
        GLUT_KEY_PAGE_UP,
        GLUT_KEY_PAGE_DOWN,
        GLUT_KEY_HOME,
        GLUT_KEY_END,
        GLUT_KEY_INSERT,
    GLUT_KEY_TAIL,

    GLUT_GET_HEAD = 2000,
        GLUT_WINDOW_X,
        GLUT_WINDOW_Y,
        GLUT_WINDOW_WIDTH,
        GLUT_WINDOW_HEIGHT,
        GLUT_WINDOW_BUFFER_SIZE,
        GLUT_WINDOW_STENCIL_SIZE,
        GLUT_WINDOW_DEPTH_SIZE,
        GLUT_WINDOW_RED_SIZE,
        GLUT_WINDOW_GREEN_SIZE,
        GLUT_WINDOW_BLUE_SIZE,
        GLUT_WINDOW_ALPHA_SIZE,
        GLUT_WINDOW_ACCUM_RED_SIZE,
        GLUT_WINDOW_ACCUM_GREEN_SIZE,
        GLUT_WINDOW_ACCUM_BLUE_SIZE,
        GLUT_WINDOW_ACCUM_ALPHA_SIZE,
        GLUT_WINDOW_DOUBLEBUFFER,
        GLUT_WINDOW_RGBA,
        GLUT_WINDOW_PARENT,
        GLUT_WINDOW_NUM_CHILDREN,
        GLUT_WINDOW_COLORMAP_SIZE,
        GLUT_WINDOW_NUM_SAMPLES,
        GLUT_WINDOW_STEREO,
        GLUT_WINDOW_CURSOR,
        GLUT_SCREEN_WIDTH,
        GLUT_SCREEN_HEIGHT,
        GLUT_SCREEN_WIDTH_MM,
        GLUT_SCREEN_HEIGHT_MM,
        GLUT_MENU_NUM_ITEMS,
        GLUT_DISPLAY_MODE_POSSIBLE,
        GLUT_INIT_WINDOW_X,
        GLUT_INIT_WINDOW_Y,
        GLUT_INIT_WINDOW_WIDTH,
        GLUT_INIT_WINDOW_HEIGHT,
        GLUT_INIT_DISPLAY_MODE,
        GLUT_ELAPSED_TIME,
        GLUT_WINDOW_FORMAT_ID,
        GLUT_INIT_STATE,

        GLUT_ACTION_ON_WINDOW_CLOSE,
        GLUT_WINDOW_BORDER_WIDTH,
        GLUT_WINDOW_HEADER_HEIGHT,
        GLUT_VERSION,
        GLUT_RENDERING_CONTEXT,
        OPENGLEAN_VERSION,

        GLUT_INIT_AUX_BUFFERS,
    GLUT_GET_TAIL,

    GLUT_DEVICE_GET_HEAD = 3000,
        GLUT_HAS_KEYBOARD,
        GLUT_HAS_MOUSE,
#ifndef GLEANDEV_H
        GLUT_HAS_SPACEBALL,
        GLUT_HAS_DIAL_AND_BUTTON_BOX,
        GLUT_HAS_TABLET,
        GLUT_NUM_MOUSE_BUTTONS,
        GLUT_NUM_SPACEBALL_BUTTONS,
        GLUT_NUM_BUTTON_BOX_BUTTONS,
        GLUT_NUM_DIALS,
        GLUT_NUM_TABLET_BUTTONS,
        GLUT_DEVICE_IGNORE_KEY_REPEAT,
        GLUT_DEVICE_KEY_REPEAT,
        GLUT_HAS_JOYSTICK,
        GLUT_OWNS_JOYSTICK,
        GLUT_JOYSTICK_BUTTONS,
        GLUT_JOYSTICK_AXES,
        GLUT_JOYSTICK_POLL_RATE,
#endif
    GLUT_DEVICE_GET_TAIL,

    GLUT_LAYER_GET_HEAD = 4000,
        GLUT_OVERLAY_POSSIBLE,
        GLUT_LAYER_IN_USE,
        GLUT_HAS_OVERLAY,
        GLUT_TRANSPARENT_INDEX,
        GLUT_NORMAL_DAMAGED,
        GLUT_OVERLAY_DAMAGED,
    GLUT_LAYER_GET_TAIL,

    GLUT_VIDEO_RESIZE_GET_HEAD = 5000,
        GLUT_VIDEO_RESIZE_POSSIBLE,
        GLUT_VIDEO_RESIZE_IN_USE,
        GLUT_VIDEO_RESIZE_X_DELTA,
        GLUT_VIDEO_RESIZE_Y_DELTA,
        GLUT_VIDEO_RESIZE_WIDTH_DELTA,
        GLUT_VIDEO_RESIZE_HEIGHT_DELTA,
        GLUT_VIDEO_RESIZE_X,
        GLUT_VIDEO_RESIZE_Y,
        GLUT_VIDEO_RESIZE_WIDTH,
        GLUT_VIDEO_RESIZE_HEIGHT,
    GLUT_VIDEO_RESIZE_GET_TAIL,

    GLUT_SET_CURSOR_HEAD = 6000,
        GLUT_CURSOR_RIGHT_ARROW,
        GLUT_CURSOR_LEFT_ARROW,
        GLUT_CURSOR_INFO,
        GLUT_CURSOR_DESTROY,
        GLUT_CURSOR_HELP,
        GLUT_CURSOR_CYCLE,
        GLUT_CURSOR_SPRAY,
        GLUT_CURSOR_WAIT,
        GLUT_CURSOR_TEXT,
        GLUT_CURSOR_CROSSHAIR,
        GLUT_CURSOR_UP_DOWN,
        GLUT_CURSOR_LEFT_RIGHT,
        GLUT_CURSOR_TOP_SIDE,
        GLUT_CURSOR_BOTTOM_SIDE,
        GLUT_CURSOR_LEFT_SIDE,
        GLUT_CURSOR_RIGHT_SIDE,
        GLUT_CURSOR_TOP_LEFT_CORNER,
        GLUT_CURSOR_TOP_RIGHT_CORNER,
        GLUT_CURSOR_BOTTOM_RIGHT_CORNER,
        GLUT_CURSOR_BOTTOM_LEFT_CORNER,
        GLUT_CURSOR_INHERIT,
        GLUT_CURSOR_NONE,
        GLUT_CURSOR_FULL_CROSSHAIR,
    GLUT_SET_CURSOR_TAIL,

    GLUT_GAME_MODE_HEAD = 7000,
        GLUT_GAME_MODE_ACTIVE,
        GLUT_GAME_MODE_POSSIBLE,
        GLUT_GAME_MODE_WIDTH,
        GLUT_GAME_MODE_HEIGHT,
        GLUT_GAME_MODE_PIXEL_DEPTH,
        GLUT_GAME_MODE_REFRESH_RATE,
        GLUT_GAME_MODE_DISPLAY_CHANGED,
    GLUT_GAME_MODE_TAIL,

    GLUT_COLOR_COMP_HEAD = 8000,
        GLUT_RED,
        GLUT_GREEN,
        GLUT_BLUE,
    GLUT_COLOR_COMP_TAIL,

    /*
     * GLUT API definitions -- mouse state definitions
     * Because these are all generally received back from
     * the API via callbacks, it is desirable that the values
     * bound to 0 remain bound to 0 so that the client can
     * treat the "button state" (for example) as a boolean
     * rather than an enumerated type.  The mouse button names are
     * somewhat stuck with us, and in any case the numbering is
     * set in stone so that more-than-3-buttons has a logical
     * extension (..., 3, 4, ...).
     *
     * The use of these macros is dubious.  Mouse up/down state
     * might better be range (indicating pressure), while the button
     * names obviously are short-sighted.  If I have 4 buttons, which
     * is GLUT_MIDDLE_BUTTON?  If I have 5, is the GLUT_LEFT_BUTTON
     * the extreme left, or the one left-of-center-but-nearest-center?
     */
    GLUT_LEFT_BUTTON   = 0,
    GLUT_MIDDLE_BUTTON,
    GLUT_RIGHT_BUTTON,
    GLUT_DOWN          = 0,
    GLUT_UP,
    GLUT_LEFT          = 0,
    GLUT_ENTERED,

    /*
     * GLUT API definitions -- windows and menu related definitions
     *
     * Some of the same notes apply here as for mouse state info.
     * The hidden/fully-retained/partially-retained/fully-covered
     * set makes some sense, and perhaps should be moved off to its
     * own sub-range in this enum.  The others are of questionable
     * merit.
     */
    GLUT_MENU_NOT_IN_USE   = 0,
    GLUT_MENU_IN_USE,
    GLUT_NOT_VISIBLE       = 0,
    GLUT_VISIBLE,
    GLUT_HIDDEN            = 0,
    GLUT_FULLY_RETAINED,
    GLUT_PARTIALLY_RETAINED,
    GLUT_FULLY_COVERED,

   /*
    * GLUT API definitions -- the glutUseLayer parameters
    *
    * Either the "overlay" should be an ID, if allowing multiple
    * overlays (in which case {GLUT_OVERLAY} is a poor name), or
    * the client should treat the overlay as a boolean value, I
    * think.
    */
    GLUT_NORMAL  = 0,
    GLUT_OVERLAY,

    /* GLUT API definitions -- additional keyboard definitions */
    GLUT_KEY_REPEAT_OFF    = 0,
    GLUT_KEY_REPEAT_ON,
    GLUT_KEY_REPEAT_DEFAULT,

};

/*
 * The display mode definitions.
 *
 * These are bitflags, of course, so don't put in the enum.
 *
 * XXX Consider deprecating the bitflag approach.
 */
#define GLUT_RGB          0x0000
#define GLUT_RGBA         0x0000
#define GLUT_INDEX        0x0001
#define GLUT_SINGLE       0x0000
#define GLUT_DOUBLE       0x0002
#define GLUT_ACCUM        0x0004
#define GLUT_ALPHA        0x0008
#define GLUT_DEPTH        0x0010
#define GLUT_STENCIL      0x0020
#define GLUT_MULTISAMPLE  0x0080
#define GLUT_STEREO       0x0100
#define GLUT_LUMINANCE    0x0200

#define GLUT_BORDERLESS   0x0800 // ML: borderless window style as planned in the docs

#ifndef GLEANFONTS_H
/*
 * GLUT API macro definitions -- fonts definitions
 *
 * {GLUTfont} is an abstraction to let us redefine the type.
 */
typedef void *GLUTfont;
#define GLUT_STROKE_ROMAN           ((GLUTfont) 0)
#define GLUT_STROKE_MONO_ROMAN      ((GLUTfont) 1)
#define GLUT_BITMAP_9_BY_15         ((GLUTfont) 2)
#define GLUT_BITMAP_8_BY_13         ((GLUTfont) 3)
#define GLUT_BITMAP_TIMES_ROMAN_10  ((GLUTfont) 4)
#define GLUT_BITMAP_TIMES_ROMAN_24  ((GLUTfont) 5)
#define GLUT_BITMAP_HELVETICA_10    ((GLUTfont) 6)
#define GLUT_BITMAP_HELVETICA_12    ((GLUTfont) 7)
#define GLUT_BITMAP_HELVETICA_18    ((GLUTfont) 8)
#endif

/*
 * GLUT API macro definitions -- the glutGetModifiers parameters
 */
#define GLUT_ACTIVE_SHIFT  0x0001
#define GLUT_ACTIVE_CTRL   0x0002
#define GLUT_ACTIVE_ALT    0x0004


#ifndef GLEANDEV_H
/*
 * GLUT API macro definitions -- joystick definitions
 */
#define GLUT_JOYSTICK_BUTTON_A  0x0001
#define GLUT_JOYSTICK_BUTTON_B  0x0002
#define GLUT_JOYSTICK_BUTTON_C  0x0004
#define GLUT_JOYSTICK_BUTTON_D  0x0008
#endif


/*
 * Initialization functions, see init.c
 */
OGAPI void OGAPIENTRY glutInit (int *pargc, char **argv);
OGAPI void OGAPIENTRY glutInitWindowPosition (int x, int y);
OGAPI void OGAPIENTRY glutInitWindowSize (int width, int height);
OGAPI void OGAPIENTRY glutInitDisplayMode (unsigned int displayMode);
OGAPI void OGAPIENTRY glutInitDisplayString (const char *displayMode);

/*
 * Process loop function, see main.c
 */
OGAPI void OGAPIENTRY glutMainLoop (void);

/*
 * Window management functions, see window.c
 */
OGAPI int OGAPIENTRY glutCreateWindow (const char *title);
OGAPI int OGAPIENTRY glutCreateSubWindow (
    int window, int x, int y, int width, int height
);
OGAPI void OGAPIENTRY glutDestroyWindow (int window);
OGAPI void OGAPIENTRY glutSetWindow (int window);
OGAPI int  OGAPIENTRY glutGetWindow (void);
OGAPI void OGAPIENTRY glutSetWindowTitle (const char *title);
OGAPI void OGAPIENTRY glutSetIconTitle (const char *title);
OGAPI void OGAPIENTRY glutReshapeWindow (int width, int height);
OGAPI void OGAPIENTRY glutPositionWindow (int x, int y);
OGAPI void OGAPIENTRY glutShowWindow (void);
OGAPI void OGAPIENTRY glutHideWindow (void);
OGAPI void OGAPIENTRY glutIconifyWindow (void);
OGAPI void OGAPIENTRY glutPushWindow (void);
OGAPI void OGAPIENTRY glutPopWindow (void);
OGAPI void OGAPIENTRY glutFullScreen (void);

/*
 * Display-connected functions, see display.c
 */
OGAPI void OGAPIENTRY glutPostWindowRedisplay (int window);
OGAPI void OGAPIENTRY glutPostRedisplay (void);
OGAPI void OGAPIENTRY glutSwapBuffers (void);

/*
 * Mouse cursor functions, see cursor.c
 */
OGAPI void OGAPIENTRY glutWarpPointer (int x, int y);
OGAPI void OGAPIENTRY glutSetCursor (int cursor);

/*
 * Overlay stuff, see overlay.c
 */
OGAPI void OGAPIENTRY glutEstablishOverlay (void);
OGAPI void OGAPIENTRY glutRemoveOverlay (void);
OGAPI void OGAPIENTRY glutUseLayer (GLenum layer);
OGAPI void OGAPIENTRY glutPostOverlayRedisplay (void);
OGAPI void OGAPIENTRY glutPostWindowOverlayRedisplay (int window);
OGAPI void OGAPIENTRY glutShowOverlay (void);
OGAPI void OGAPIENTRY glutHideOverlay (void);

/*
 * Menu stuff, see menu.c
 */
OGAPI int  OGAPIENTRY glutCreateMenu (void (*callback) (int menu));
OGAPI void OGAPIENTRY glutDestroyMenu (int menu);
OGAPI int  OGAPIENTRY glutGetMenu (void);
OGAPI void OGAPIENTRY glutSetMenu (int menu);
OGAPI void OGAPIENTRY glutAddMenuEntry (const char *label, int value);
OGAPI void OGAPIENTRY glutAddSubMenu (const char *label, int subMenu);
OGAPI void OGAPIENTRY glutChangeToMenuEntry (
    int item, const char *label, int value
);
OGAPI void OGAPIENTRY glutChangeToSubMenu (
    int item, const char *label, int value
);
OGAPI void OGAPIENTRY glutRemoveMenuItem (int item);
OGAPI void OGAPIENTRY glutAttachMenu (int button);
OGAPI void OGAPIENTRY glutDetachMenu (int button);

/*
 * Global callback functions, see callbacks.c
 */
OGAPI void OGAPIENTRY glutTimerFunc (
    unsigned int time, void (*callback) (int data), int value
);
OGAPI void OGAPIENTRY glutIdleFunc (void (*callback) (void));

/*
 * Window-specific callback functions, see callbacks.c
 */
OGAPI void OGAPIENTRY glutKeyboardFunc (
    void (*callback) (unsigned char key, int x, int y)
);
OGAPI void OGAPIENTRY glutSpecialFunc (
    void (*callback) (int key, int x, int y)
    );
OGAPI void OGAPIENTRY glutReshapeFunc (void (*callback) (int w, int x));
OGAPI void OGAPIENTRY glutVisibilityFunc (void (*callback) (int status));
OGAPI void OGAPIENTRY glutDisplayFunc (void (*callback) (void));
OGAPI void OGAPIENTRY glutMouseFunc (
    void (*callback) (int button, int state, int x, int y)
);
OGAPI void OGAPIENTRY glutMotionFunc (void (*callback) (int x, int y));
OGAPI void OGAPIENTRY glutPassiveMotionFunc (
    void (*callback) (int x, int y)
);
OGAPI void OGAPIENTRY glutEntryFunc (void (*callback) (int state));

OGAPI void OGAPIENTRY glutKeyboardUpFunc (
    void (*callback) (unsigned char key, int x, int y)
);
OGAPI void OGAPIENTRY glutSpecialUpFunc (
    void (*callback) (int key, int x, int y)
);
#ifndef GLEANDEV_H
OGAPI void OGAPIENTRY glutJoystickFunc (
    void (*callback) (
        unsigned int buttons, int xaxis, int yaxis, int zaxis
    ), int pollInterval
);
#endif
OGAPI void OGAPIENTRY glutMenuStatusFunc (
    void (*callback) (int status, int x, int y)
);
OGAPI void OGAPIENTRY glutOverlayDisplayFunc (void (*callback) (void));
OGAPI void OGAPIENTRY glutWindowStatusFunc (void (*callback) (int state));
#ifndef GLEANDEV_H
OGAPI void OGAPIENTRY glutSpaceballMotionFunc (
    void (*callback) (int x, int y, int z)
);
OGAPI void OGAPIENTRY glutSpaceballRotateFunc (
    void (*callback) (int x, int y, int z)
);
OGAPI void OGAPIENTRY glutSpaceballButtonFunc (
    void (*callback) (int button, int state)
);
OGAPI void OGAPIENTRY glutButtonBoxFunc (
    void (*callback) (int button, int state)
);
OGAPI void OGAPIENTRY glutDialsFunc (
    void (*callback) (int dial, int value)
);
OGAPI void OGAPIENTRY glutTabletMotionFunc (
    void (*callback) (int x, int y)
);
OGAPI void OGAPIENTRY glutTabletButtonFunc (
    void (*callback) (int button, int state, int x, int y)
);
#endif

/*
 * State setting and retrieval functions, see state.c
 *
 * XXX Maybe make glutDeviceGet() into a gleandev function?
 */
OGAPI int OGAPIENTRY glutGet (GLenum query);
OGAPI int OGAPIENTRY glutDeviceGet (GLenum query);
OGAPI int OGAPIENTRY glutGetModifiers (void);
OGAPI int OGAPIENTRY glutLayerGet (GLenum query);

#ifndef GLEANFONTS_H
/*
 * Font stuff, see font.c
 */
OGAPI void  OGAPIENTRY glutBitmapCharacter (GLUTfont font, int character);
OGAPI int   OGAPIENTRY glutBitmapWidth (GLUTfont font, int character);
OGAPI void  OGAPIENTRY glutStrokeCharacter (GLUTfont font, int character);
OGAPI float OGAPIENTRY glutStrokeWidth (GLUTfont font, int character);
OGAPI int   OGAPIENTRY glutBitmapLength (
    GLUTfont font, const unsigned char *string
);
OGAPI float OGAPIENTRY glutStrokeLength (
    GLUTfont font, const unsigned char *string
);
#endif


#ifndef GLEANSHAPES_H
/*
 * Geometry functions, see geometry.c
 */
OGAPI void OGAPIENTRY glutWireCube (GLdouble size);
OGAPI void OGAPIENTRY glutSolidCube (GLdouble size);
OGAPI void OGAPIENTRY glutWireCuboctahedron (GLdouble s);
OGAPI void OGAPIENTRY glutSolidCuboctahedron (GLdouble s);
OGAPI void OGAPIENTRY glutWireSphere (
    GLdouble radius, GLint slices, GLint stacks
);
OGAPI void OGAPIENTRY glutSolidSphere (
    GLdouble radius, GLint slices, GLint stacks
);
OGAPI void OGAPIENTRY glutWireCone (
    GLdouble base, GLdouble height, GLint slices, GLint stacks
);
OGAPI void OGAPIENTRY glutSolidCone (
    GLdouble base, GLdouble height, GLint slices, GLint stacks
);

OGAPI void OGAPIENTRY glutWireTorus (
    GLdouble innerRadius, GLdouble outerRadius, GLint sides, GLint rings
);
OGAPI void OGAPIENTRY glutSolidTorus (
    GLdouble innerRadius, GLdouble outerRadius, GLint sides, GLint rings
);
OGAPI void OGAPIENTRY glutWireDodecahedron (void);
OGAPI void OGAPIENTRY glutSolidDodecahedron (void);
OGAPI void OGAPIENTRY glutWireOctahedron (void);
OGAPI void OGAPIENTRY glutSolidOctahedron (void);
OGAPI void OGAPIENTRY glutWireTetrahedron (void);
OGAPI void OGAPIENTRY glutSolidTetrahedron (void);
OGAPI void OGAPIENTRY glutWireIcosahedron (void);
OGAPI void OGAPIENTRY glutSolidIcosahedron (void);

/*
 * Teapot rendering functions, found in teapot.c
 */
OGAPI void OGAPIENTRY glutWireTeapot (GLdouble size);
OGAPI void OGAPIENTRY glutSolidTeapot (GLdouble size);

#endif


/*
 * Game mode functions, see gamemode.c
 */
OGAPI void OGAPIENTRY glutGameModeString (const char *string);
OGAPI int  OGAPIENTRY glutEnterGameMode (void);
OGAPI void OGAPIENTRY glutLeaveGameMode (void);
OGAPI int  OGAPIENTRY glutGameModeGet (GLenum query);

/*
 * Video resize functions, see videoresize.c
 */
OGAPI int  OGAPIENTRY glutVideoResizeGet (GLenum query);
OGAPI void OGAPIENTRY glutSetupVideoResizing (void);
OGAPI void OGAPIENTRY glutStopVideoResizing (void);
OGAPI void OGAPIENTRY glutVideoResize (int x, int y, int width, int height);
OGAPI void OGAPIENTRY glutVideoPan (int x, int y, int width, int height);

/*
 * Colormap functions, see misc.c
 */
OGAPI void    OGAPIENTRY glutSetColor (
    int color, GLfloat red, GLfloat green, GLfloat blue
);
OGAPI GLfloat OGAPIENTRY glutGetColor (int color, int component);
OGAPI void    OGAPIENTRY glutCopyColormap (int window);

/*
 * Misc keyboard and joystick functions, see misc.c
 */
OGAPI void OGAPIENTRY glutIgnoreKeyRepeat (int ignore);
OGAPI void OGAPIENTRY glutSetKeyRepeat (int repeatMode);

#ifndef GLEANDEV_H
OGAPI void OGAPIENTRY glutForceJoystickFunc (void);

/*
 * These remaining joystick functions are ad-hoc creations
 * of freeglut.  They are not very complete nor well-designed,
 * but it seems that freeglut intends to export them in the
 * next release.
 *
 * In order to keep OpenGLEAN abreast of that, I'm including
 * them formally, now, in OpenGLEAN.  Besides...I always intended
 * to expose them properly once I separated the joystick code
 * from the core.  So...it's not much more trouble to do it
 * now.  I may round the set out a bit further.
 */
OGAPI int OGAPIENTRY glutGetJoystickNumAxes (int ident);
OGAPI int OGAPIENTRY glutGetJoystickNumButtons (int ident);
OGAPI int OGAPIENTRY glutGetJoystickNotWorking (int ident);
OGAPI float OGAPIENTRY glutGetJoystickDeadBand (int ident, int axis);
OGAPI void OGAPIENTRY glutSetJoystickDeadBand (int ident, int axis, float db);
OGAPI float OGAPIENTRY glutGetJoystickSaturation (int ident, int axis);
OGAPI void OGAPIENTRY glutSetJoystickSaturation (int ident, int axis, float st);
OGAPI void OGAPIENTRY glutSetJoystickMinRange (int ident, float *axes);
OGAPI void OGAPIENTRY glutSetJoystickMaxRange (int ident, float *axes);
OGAPI void OGAPIENTRY glutSetJoystickCenter (int ident, float *axes);
OGAPI void OGAPIENTRY glutGetJoystickMinRange (int ident, float *axes);
OGAPI void OGAPIENTRY glutGetJoystickMaxRange (int ident, float *axes);
OGAPI void OGAPIENTRY glutGetJoystickCenter (int ident, float *axes);
#endif


/*
 * Misc functions, see misc.c
 */
OGAPI int  OGAPIENTRY glutExtensionSupported (const char *extension);
OGAPI void OGAPIENTRY glutReportErrors (void);




/*
 * This marks the boundary between
 * the old openglean_std.h (above) and
 * the old openglean_ext.h (below).
 *
 * The arbitrary, historic division between these two
 * parts of the normal OpenGLEAN header should be merged.
 */



/*
 * GLUT API Extension macro definitions -- behavior when the user clicks
 * on the window-close/program-quit widget box.
 */
#define GLUT_ACTION_EXIT                         0
#define GLUT_ACTION_GLUTMAINLOOP_RETURNS         1
#define GLUT_ACTION_CONTINUE_EXECUTION           2

/*
 * Create a new rendering context when the user opens a new window?
 */
#define GLUT_CREATE_NEW_CONTEXT                  0
#define GLUT_USE_CURRENT_CONTEXT                 1

/*
 * GLUT API Extension macro definitions -- display mode
 */
#define GLUT_OFFSCREEN                      0x0400

/*
 * Process loop function, see main.c
 */
OGAPI void OGAPIENTRY glutMainLoopEvent (void);
OGAPI void OGAPIENTRY glutLeaveMainLoop (void);

/*
 * Window-specific callback functions, see callbacks.c
 */
OGAPI void OGAPIENTRY glutMouseWheelFunc (
    void (*callback) (int wheel, int direction, int x, int y)
);
OGAPI void OGAPIENTRY glutCloseFunc (void (*callback) (void));
OGAPI void OGAPIENTRY glutWMCloseFunc (void (*callback) (void));
OGAPI void OGAPIENTRY glutMenuDestroyFunc (void (*callback) (void));

/*
 * State setting and retrieval functions, see state.c
 */
OGAPI void  OGAPIENTRY glutSetOption (GLenum option_flag, int value);
OGAPI void *OGAPIENTRY glutGetWindowData (void);
OGAPI void  OGAPIENTRY glutSetWindowData (void *data);
OGAPI void *OGAPIENTRY glutGetMenuData (void);
OGAPI void  OGAPIENTRY glutSetMenuData (void *data);


#ifndef GLEANFONTS_H
/*
 * Font stuff, see font.c
 */
OGAPI int     OGAPIENTRY glutBitmapHeight (GLUTfont font);
OGAPI GLfloat OGAPIENTRY glutStrokeHeight (GLUTfont font);
OGAPI void    OGAPIENTRY glutBitmapString (
    GLUTfont font, const unsigned char *string
);
OGAPI void    OGAPIENTRY glutStrokeString (
    GLUTfont font, const unsigned char *string
);
#endif

#ifndef GLEANSHAPES_H
/*
 * Geometry functions, see geometry.c
 */
OGAPI void OGAPIENTRY glutWireRhombicDodecahedron (void);
OGAPI void OGAPIENTRY glutSolidRhombicDodecahedron (void);
OGAPI void OGAPIENTRY glutWireSierpinskiSponge (
    int num_levels, const GLdouble offset [3], GLdouble scale
);
OGAPI void OGAPIENTRY glutSolidSierpinskiSponge (
    int num_levels, const GLdouble offset [3], GLdouble scale
);
OGAPI void OGAPIENTRY glutWireCylinder (
    GLdouble radius, GLdouble height, GLint slices, GLint stacks
);
OGAPI void OGAPIENTRY glutSolidCylinder (
    GLdouble radius, GLdouble height, GLint slices, GLint stacks
);
#endif

/*
 * Extension functions, see ext.c
 */
OGAPI void *OGAPIENTRY glutGetProcAddress (const char *procName);


#ifdef __cplusplus
}
#endif

#endif
