/*!
    \file  globals.c
    \brief OpenGLEAN global variables
*/

/*
**  Copyright 2005, the OpenGLEAN project.
**
**  This file contains the OpenGLEAN general globals.
**  (Excluded from this is the font data.  There are
**   some good reasons to keep it separate: The font
**   data is machine-generated, not readily managed
**   by hand, and only of use to the font.c file.)
**
*/


/*
**  This structure holds all of the information related to
**  the display, screen, root window, etc.
**
**  Historical note: Originally declared in init.c
**
*/
PRIVATE SOG_Display ogDisplay;

/*
**  Current OpenGLEAN general state.
**
**  Historical note: Originally declared in init.c.
**
*/
PRIVATE SOG_State ogState =
{
    // Init value                            Field name
    {-1, -1, GL_FALSE},                   // Position
    {300, 300, GL_TRUE},                  // Size
    GLUT_RGBA | GLUT_SINGLE | GLUT_DEPTH, // DisplayMode
    0,                                    // # of Aux buffers
    GL_FALSE,                             // Initialised
    GL_FALSE,                             // ForceDirectContext
    GL_TRUE,                              // TryDirectContext
    GL_FALSE,                             // ForceIconic
    GL_FALSE,                             // UseCurrentContext
    GL_FALSE,                             // GLDebugSwitch
    GL_TRUE,                              // PrintErrors
    GL_TRUE,                              // PrintWarnings
    GL_FALSE,                             // PrintInforms
    GL_FALSE,                             // XSyncSwitch
    GLUT_KEY_REPEAT_ON,                   // KeyRepeat
    0xffffffff,                           // Modifiers
    0,                                    // FPSInterval
    0,                                    // SwapCount
    0,                                    // SwapTime
#if TARGET_HOST_UNIX_X11
    {{0, 0}, GL_FALSE},                   // Time
#elif TARGET_HOST_WIN32 || TARGET_HOST_WINCE
    {0, GL_FALSE},                        // Time
#endif
    {NULL, NULL},                         // Timers
    {NULL, NULL},                         // FreeTimers
    NULL,                                 // IdleCallback
    0,                                    // ActiveMenus
    NULL,                                 // MenuStatusCallback
    {640, 40, GL_TRUE},                   // GameModeSize
    16,                                   // GameModeDepth
    72,                                   // GameModeRefresh
    GLUT_ACTION_EXIT,                     // ActionOnWindowClose
    GLUT_EXEC_STATE_INIT,                 // ExecState
    NULL,                                 // ProgramName
    GL_FALSE,                             // JoysticksInitted
    NULL,                                 // BackToMainLoop
    0,                                    // InMainLoop
    0,                                    // ButtonMask
};


/*
**  This structure contains data for windows & menus created between
**  glutInit() being called and glutMainLoop() returning.
**  (A little more generally and accurately, "while OpenGLEAN
**   is alive", albeit the term "alive" a little non-technical.)
**
**
**  Historical note: Originally declared in  structure.c
*/
PRIVATE SOG_Structure ogStructure =
{
    // Value            Field name
    {NULL, NULL},    // Windows
    {NULL, NULL},    // Menus
    {NULL, NULL},    // WindowsToDestroy
    NULL,            // Window
    NULL,            // Menu
    NULL,            // MenuContext
    NULL,            // GameMode
    0,               // WindowID
    0,               // MenuID
};
