/*!
    \file  openglean.c
    \brief One-stop compilation.
*/

/*
**  Copyright 2004, 2005 the OpenGLEAN Project.
**
**  The OpenGLEAN "master source file".
**
**  This slightly unusual technique is used because OpenGLEAN
**  has inherited its modularization structure from OpenGLUT
**  and freeglut.  Those libraries have to produce many non-{static}
**  symbols in order to export the symbols between the various *.c
**  files.  One solution to that is to redesign the code.  Another
**  solution is to do something like this.  (Re-design is
**  arguably better, but often design ideals run into practical
**  limitations.  Besides which, undesired design decisions are very
**  hard to undo late in life.)
**
**  An added advantage of this is that compilation speed should be
**  higher, at least for the end-user, as the compiler won't need
**  to re-read, re-parse, and re-evaluate all of the headers common
**  to so much of OpenGLEAN.
**
*/

/*
** Get the results of running ./configure, if available.
**
*/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

/*
**  "internal.h" defines the {PRIVATE} symbol to nothing if it is
**  not defined upon entry.  Presently, OpenGLEAN allows you to
**  define it as an empty string in order to re-expose all of
**  the private symbols that OpenGLUT 0.6.3 exposed.
*/
#ifndef PRIVATE
#define PRIVATE static
#endif

/*
**  The general, private OpenGLEAN header.  Currently, most
**  .c files include this also, but we might as well drop that
**  and save a little compilation time: Include it once.
**
*/
#include "internal.h"

/*
**  Next, ensure that our (non-font) global data is visible.
**
*/
#include "globals.c"

/*
**  The font data files must preceed the font.c file.
**  (One way to address this is to #include the font data
**   into the font.c file...  Another would be to chain it
**   into the globals.c)
**
*/
#include "font_data.c"
#include "stroke_mono_roman.c"
#include "stroke_roman.c"

/*
**  Now, our general sources:
**
*/
#include "callbacks.c"
#include "colormap.c"
#include "cursor.c"
#include "display.c"
#include "ext.c"
#include "font.c"
#include "gamemode.c"
#include "geometry.c"
#include "init.c"
#include "joystick.c"
#include "main.c"
#include "menu.c"
#include "misc.c"
#include "overlay.c"
#include "state.c"
#include "structure.c"
#include "teapot.c"
#include "videoresize.c"
#include "window.c"
#include "winsys.c"
