#ifdef _DEBUG
#include <crtdbg.h>
#endif

#ifdef WITH_BLENDMASK
#include <GL/openglean.h>
#include "DaveBlendMask_PGM/PgmImage.h"
//#include "DaveBlendMask_YImage/YImage.hpp"
#endif // WITH_BLENDMASK

#include "Dave.h"

#include "DaveConfig.h"
//#include "DaveWindowManager.h"
#include "DaveProjection.h"
#include "DaveNetwork.h"
#include "DaveConfig.h"

#include "DaveTracking.h"

#include <iostream>
#include <stdlib.h>

DAVE::Dave::Dave() {
	#ifdef _DEBUG
		_ASSERTE(_CrtCheckMemory());
	#endif

	tIsServer = false;
	tIsClient = false;
	config = new DaveConfig();
	windowManager = NULL;
	networkClient = NULL;
	networkServer = NULL;

	syncBufferUserData = NULL;
	syncBuffer = NULL;
	syncBufferSize = 0;

	tracking = NULL;

	// read environment variables
	char* instName = getenv("DAVE_INSTANCE_NAME");
	if (instName==NULL) {
		printf("Environment variable DAVE_INSTANCE_NAME was empty!\n");
	} else {
		config->setValue("InstanceName", instName);
	}

	char* confFileName = getenv("DAVE_CONFIG_FILE");
	if (confFileName == NULL) {
		printf("Environment variable DAVE_CONFIG_FILE was empty!\n");
	} else {
		config->processConfigFile(confFileName);
	}

	#ifdef _DEBUG 
		_ASSERTE(_CrtCheckMemory());
	#endif
}

DAVE::Dave::~Dave() {
	#ifdef _DEBUG
		_ASSERTE(_CrtCheckMemory());
	#endif

	if (windowManager) delete windowManager;
	if (networkServer) delete networkServer;
	if (networkClient) delete networkClient;
	if (tracking) delete tracking;
	if (syncBuffer) delete[] syncBuffer;

	#ifdef _DEBUG
		_ASSERTE(_CrtCheckMemory());
	#endif
}

void DAVE::Dave::parseCmdLine(char argc, char* argv[]) {
	config->processCmdLineArguments(argc, argv);
}

void DAVE::Dave::init(unsigned int userDataSize/* = 0*/) {
	#ifdef _DEBUG
		_ASSERTE(_CrtCheckMemory());
	#endif

//config->printAll();
	std::string iName = config->getString("InstanceName");
	std::cout << "Instance: " << iName << std::endl;

	tIsServer = ((iName == "master") || (iName == ""));
	tIsClient = !tIsServer;

	#ifdef _DEBUG
		_ASSERTE(_CrtCheckMemory());
	#endif
	// get memory for network synchronized data (including user data)
	syncBufferSize = sizeof(SyncBufferStruct) + userDataSize;
	syncBuffer = (SyncBufferStruct*) new unsigned char[syncBufferSize];
	if (userDataSize) syncBufferUserData = (unsigned char*) syncBuffer + sizeof(SyncBufferStruct);

	syncBuffer->magicNumber[0] = 'D';
	syncBuffer->magicNumber[1] = 'A';
	syncBuffer->magicNumber[2] = 'V';
	syncBuffer->magicNumber[3] = 'E';
	syncBuffer->eyePosLeft[0] = -0.031;
	syncBuffer->eyePosLeft[1] = 0;
	syncBuffer->eyePosLeft[2] = 0;
	syncBuffer->eyePosRight[0] = 0.031;
	syncBuffer->eyePosRight[1] = 0;
	syncBuffer->eyePosRight[2] = 0;

	#ifdef _DEBUG
		_ASSERTE(_CrtCheckMemory());
	#endif

	windowManager = new DaveWindowManager(config);

	#ifdef _DEBUG
		_ASSERTE(_CrtCheckMemory());
	#endif

	if (tIsServer) {
		networkServer = getDaveNetworkServer();
		tracking = getDaveTracking();
	}
	if (tIsClient) {
		networkClient = getDaveNetworkClient();
	}

	#ifdef _DEBUG
		_ASSERTE(_CrtCheckMemory());
	#endif
}

void DAVE::Dave::update() {
	#ifdef _DEBUG
		_ASSERTE(_CrtCheckMemory());
	#endif

	// server
	if (tracking) tracking->update();
	if (networkServer) {
		if (tracking) for (int i=0; i<3; i++) {
			syncBuffer->eyePosLeft[i] = tracking->eyePosLeft[i];
			syncBuffer->eyePosRight[i] = tracking->eyePosRight[i];
		}
		networkServer->sendStatusToClients((void*) syncBuffer, syncBufferSize);
		//std::cout << "sent " << syncBufferSize << " bytes" << std::endl;
	}

	// client
	if (networkClient) {
		int count = 0;
		while ((networkClient->receiveStatusFromServer((void*) syncBuffer, syncBufferSize)) && (count < 20)) count++;
		//std::cout << "received data packets:" << count << std::endl;
		if (count && (syncBuffer->magicNumber[0] != 'D') || (syncBuffer->magicNumber[1] != 'A') ||
			(syncBuffer->magicNumber[2] != 'V') || (syncBuffer->magicNumber[3] != 'E')) {
			std::cerr << "network error: received packet with wrong magic number!" << std::endl;
			// TODO: throw an exception?
		}
	}

	#ifdef _DEBUG
		_ASSERTE(_CrtCheckMemory());
	#endif
}

DAVE::DaveNetworkClientUDP* DAVE::Dave::getDaveNetworkClient() {
	#ifdef _DEBUG
		_ASSERTE(_CrtCheckMemory());
	#endif

	std::string multicastAddress = config->getString("multicastAddress");
	if (multicastAddress.empty()) {
		std::cerr << "multicastAddress missing!" << std::endl;
		return NULL;
	}
	int multicastPort = config->getInt("multicastPort");
	if (multicastPort == 0) {
		std::cerr << "multicastPort missing!" << std::endl;
		return NULL;
	}

	#ifdef _DEBUG
		_ASSERTE(_CrtCheckMemory());
	#endif

	std::cout << "multicast: listening to " << multicastAddress << ":" << multicastPort << std::endl;
	return new DAVE::DaveNetworkClientUDP(multicastAddress.c_str(), multicastPort);
}

DAVE::DaveNetworkServerUDP* DAVE::Dave::getDaveNetworkServer() {
	#ifdef _DEBUG
		_ASSERTE(_CrtCheckMemory());
	#endif

	std::string multicastAddress = config->getString("multicastAddress");
	if (multicastAddress.empty()) {
		std::cerr << "multicastAddress missing!" << std::endl;
		return NULL;
	}
	int multicastPort = config->getInt("multicastPort");
	if (multicastPort == 0) {
		std::cerr << "multicastPort missing!" << std::endl;
		return NULL;
	}

	DAVE::DaveNetworkServerUDP* s = new DAVE::DaveNetworkServerUDP(true);
	s->addClient(multicastAddress.c_str(), multicastPort);
	std::cout << "multicast: sending to " << multicastAddress << ":" << multicastPort << std::endl;

	#ifdef _DEBUG
		_ASSERTE(_CrtCheckMemory());
	#endif

	return s;
}

DAVE::DaveTracking* DAVE::Dave::getDaveTracking() {
	#ifdef _DEBUG
		_ASSERTE(_CrtCheckMemory());
	#endif

	// look for default eye position values
	syncBuffer->eyePosLeft[0] = config->getDouble("trackingDefaulEyePosLeftX", -0.031);
	syncBuffer->eyePosLeft[1] = config->getDouble("trackingDefaulEyePosLeftY", 0);
	syncBuffer->eyePosLeft[2] = config->getDouble("trackingDefaulEyePosLeftZ", 0);
	syncBuffer->eyePosRight[0] = config->getDouble("trackingDefaulEyePosRightX", 0.031);
	syncBuffer->eyePosRight[1] = config->getDouble("trackingDefaulEyePosRightY", 0);
	syncBuffer->eyePosRight[2] = config->getDouble("trackingDefaulEyePosRightZ", 0);

	// read tracking configuration
	std::string tsAddress = config->getString("trackingServer.address");
	if (tsAddress.empty()) {
		std::cerr << "trackingServer.address missing!" << std::endl;
		return NULL;
	}
	int tsPort = config->getInt("trackingServer.port");
	if (tsPort == 0) {
		std::cerr << "trackingServer.port missing!" << std::endl;
		return NULL;
	}

	DAVE::DaveTracking* t = new DAVE::DaveTracking(tsAddress.c_str(), tsPort);
	// initialize tracking data with default values (in case no tracking data arrives via network)
	for (int i=0; i<3; i++) {
		t->eyePosLeft[i] = syncBuffer->eyePosLeft[i];
		t->eyePosRight[i] = syncBuffer->eyePosRight[i];
	}

	t->update();

	#ifdef _DEBUG
		_ASSERTE(_CrtCheckMemory());
	#endif

	return t;
}

// eye position relative to head tracking target
// tracking 
void DAVE::Dave::getEyePosDaveCS(DaveWindowManager::DBufferStruct* db, float* eyeX, float* eyeY, float* eyeZ) {
	if (tracking) {
		// use tracking data directly (and not data in sync buffer)
		switch (db->cameraStereoOffset) {
			case DaveWindowManager::DBufferStruct::LEFT_EYE:
				*eyeX = tracking->eyePosLeft[0];
				*eyeY = tracking->eyePosLeft[1];
				*eyeZ = tracking->eyePosLeft[2];
				break;
			case DaveWindowManager::DBufferStruct::RIGHT_EYE:
				*eyeX = tracking->eyePosRight[0];
				*eyeY = tracking->eyePosRight[1];
				*eyeZ = tracking->eyePosRight[2];
				break;
			default: // center
				*eyeX = 0.5*(tracking->eyePosLeft[0] + tracking->eyePosRight[0]);
				*eyeY = 0.5*(tracking->eyePosLeft[1] + tracking->eyePosRight[1]);
				*eyeZ = 0.5*(tracking->eyePosLeft[2] + tracking->eyePosRight[2]);
				break;
		}
	} else {
		// try to use tracking data coming from network
		if (!syncBuffer) {
			// not initialized
			std::cerr << "davelib4 error: getEyePosDaveCS() called before init()" << std::endl;
			exit(1);
		}

		switch (db->cameraStereoOffset) {
			case DaveWindowManager::DBufferStruct::LEFT_EYE:
				*eyeX = syncBuffer->eyePosLeft[0];
				*eyeY = syncBuffer->eyePosLeft[1];
				*eyeZ = syncBuffer->eyePosLeft[2];
				break;
			case DaveWindowManager::DBufferStruct::RIGHT_EYE:
				*eyeX = syncBuffer->eyePosRight[0];
				*eyeY = syncBuffer->eyePosRight[1];
				*eyeZ = syncBuffer->eyePosRight[2];
				break;
			default: // center
				*eyeX = 0.5*(syncBuffer->eyePosLeft[0] + syncBuffer->eyePosRight[0]);
				*eyeY = 0.5*(syncBuffer->eyePosLeft[1] + syncBuffer->eyePosRight[1]);
				*eyeZ = 0.5*(syncBuffer->eyePosLeft[2] + syncBuffer->eyePosRight[2]);
				break;
		}
	}
}

void DAVE::Dave::setProjectionMatrix(DaveWindowManager::DBufferStruct* db) {
	float x, y, z;
	getEyePosDaveCS(db, &x, &y, &z);

	windowManager->windows[db->windowIndex].projection->setProjectionMatrix(x, y, z, (tracking) ? tracking->head : NULL);
}

// caution: this 'destroys' the projection matrix, but this should be the last rendering command anyway
void DAVE::Dave::applyBlendMask(void* blendMaskImgVPtr) {
#ifdef WITH_BLENDMASK
	if (!blendMaskImgVPtr) return;

//	YImage* blendMaskImg = (YImage*) blendMaskImgVPtr;
	PgmImage* blendMaskImg = (PgmImage*) blendMaskImgVPtr;
	if (!blendMaskImg->data) return;

	glViewport(0, 0, blendMaskImg->width, blendMaskImg->height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	//glOrtho(0, blendMaskImg->width(), blendMaskImg->height(), 0, -1, 1);
	glOrtho(0, blendMaskImg->width, blendMaskImg->height, 0, -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glRasterPos2i(0, 0);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glPixelZoom(1.0, -1.0);
// TODO : glDrawPixels transfers the same whole image each time! use texture instead!
	//glDrawPixels(blendMaskImg->width(), blendMaskImg->height(), GL_RGBA, GL_UNSIGNED_BYTE, blendMaskImg->data());
	glDrawPixels(blendMaskImg->width, blendMaskImg->height, GL_RGBA, GL_UNSIGNED_BYTE, blendMaskImg->data);
	glPixelZoom(1.0, 1.0);

#endif // WITH_BLENDMASK
}
