// Windows: link with davelib4.lib Ws2_32.lib


#ifndef DAVELIB_HEADER_INCLUDED
#define DAVELIB_HEADER_INCLUDED

#include "DaveWindowManager.h"

namespace DAVE {

class DaveNetworkClientUDP;
class DaveNetworkServerUDP;
class DaveConfig;

class DaveTracking;

class Dave {
  public:
	Dave();
	~Dave();

	void parseCmdLine(char argc, char* argv[]);
	void init(unsigned int userDataSize = 0); // specify additional amout of user status data to be synchronized (pointer is syncBuffer)
	void update();

	bool isServer() {return tIsServer;};
	bool isClient() {return tIsClient;};

	DaveConfig* config;
	DaveWindowManager* windowManager;
	DaveNetworkClientUDP* networkClient;
	DaveNetworkServerUDP* networkServer;

	DaveTracking* tracking;

	void setProjectionMatrix(DaveWindowManager::DBufferStruct* db);
	void applyBlendMask(void* blendMaskImgVPtr);

	void* getSyncBufferUserDataPtr() {return (void*) syncBufferUserData;};
	void getEyePosDaveCS(DaveWindowManager::DBufferStruct* db, float* eyeX, float* eyeY, float* eyeZ); // eye position in DAVE coordinate system, including tracking and stereo offset

  private:
	DaveNetworkClientUDP* getDaveNetworkClient();
	DaveNetworkServerUDP* getDaveNetworkServer();

	DaveTracking* getDaveTracking();

	bool tIsServer, tIsClient;
//#pragma pack(push, 1)
// alignment should not be problematic at the moment
	struct SyncBufferStruct {char magicNumber[4]; float eyePosLeft[3], eyePosRight[3];} *syncBuffer;
//#pragma pack(pop)
	int syncBufferSize; // size of whole buffer including used data
	unsigned char* syncBufferUserData; // only user data (rear part of daveBuffer)
};

} // namespace DAVE

#endif // DAVELIB_HEADER_INCLUDED
