// basic read and write support for PPM (1 bit black/white), PGM (8 bit gray), PBM (24 bit rgb) formats
//
// by Marcel Lancelle

#include <iostream>
#include <fstream>
#include <string>
#include <assert.h>

#include "pnm.h"

namespace DAVE {
void pnmSkipWhiteSpaces(std::ifstream* is) {
	bool isWhiteSpace;
	do {
		char c = is->peek();
		isWhiteSpace = (c == ' ') || (c == 13) || (c == 10) || (c == '\t');
		if (isWhiteSpace) is->get(c);
	} while (isWhiteSpace);
}
} // namespace DAVE

namespace DAVE {
int getPnmValue(std::ifstream* is) {
	if (!*is) return 0;

	pnmSkipWhiteSpaces(is);

	// ignore line commens starting with '#'
	while (*is && (is->peek() == '#')) {
		is->ignore(9999, '\n');
	}

	pnmSkipWhiteSpaces(is);

	// *is >> i;

	// read integer
	int i = 0;
	char buf[10];

	while (is && (i<9)) {
		is->get(buf[i]);
		if ((buf[i] < '0') || (buf[i] > '9')) break;
		i++;
	}
	buf[i+1] = 0;

	int val = atoi(buf);
	return val;
}
} // namespace DAVE

////////////////////////////////////////////////////////////////////////////////////////////
// P5 - PGM (binary 8 bit gray image)
////////////////////////////////////////////////////////////////////////////////////////////

// save 
bool DAVE::savePGM(std::string fileName, int width, int height, unsigned char* data, int maxIntensity/* = 255*/) {
	std::ofstream os;

	os.open(fileName.c_str(), std::ios::out | std::ios::binary);
	if (!os) return false; // file couldn't be opened

	os << "P5" << (char) 10 << width << " " << height << (char) 10 << maxIntensity << (char) 10;

	// write image as raw data
	bool success = os.write((const char*) data, width*height);
	os.close();

	return success;
}



// read image; the necessary memory will be allocated with new, call delete[] when you don't need it any more!
bool DAVE::loadPGM(std::string fileName, int* width, int* height, unsigned char** data, int* maxIntensity/* = NULL*/) {
	std::ifstream is;

	is.open(fileName.c_str(), std::ios::in | std::ios::binary);
	if (!is.is_open()) return false; // file couldn't be opened

	char magicNumber[3] = {0, 0, 0};
	is.get(magicNumber, 3);
	if ((magicNumber[0] != 'P') || (magicNumber[1] != '5')) {
		is.close();
		return false;
	}

	*width = getPnmValue(&is);
	assert(width > 0);
	*height = getPnmValue(&is);
	assert(height > 0);
	int maxInt = getPnmValue(&is);
	assert(maxInt > 0);
	if (maxIntensity) *maxIntensity = maxInt;

	*data = new unsigned char[*width * *height];

	bool success = is.read((char*) *data, *width * *height);
	is.close();

	return success;
}
