// helper class for convenience
// there is a list with keys who have a value, each key exists only once

#ifndef DAVECONFIG_HEADER_INCLUDED
#define DAVECONFIG_HEADER_INCLUDED

#include <string>
#include <map>

namespace DAVE {

class DaveConfig {
  public:
	DaveConfig();
	~DaveConfig();

	/**
	 * for settings/overrides on command line
	 * everything is processed in order so config file settings
	 * may be overridden by following command line settings
	 * use -davecfg configFileName on the command line to invoke
	 * a call to processConfigFile()
	 */
	void processCmdLineArguments(char argc, char* argv[]);

	/**
	 * calls processLine() for each line
	 */
	bool processConfigFile(char* configFileName);

	/**
	 * first token is key, second value
	 * '=' are treated as whitespaces (so they can be used)
	 * whitespaces are ignored (before and after value)
	 * value containing whitespaces can be enclosed in "" (meaning " are ignored)
	 * key has to start with alphabetical value (ignored otherwise)
	 */
	bool processLine(std::string line);

	std::string getString(char* key);
	std::string getString(std::string key);

	double getDouble(char* key);
	double getDouble(std::string key, double defaultValue = 0);

	int getInt(char* key);
	int getInt(std::string key, int defaultValue = 0);

	bool hasKey(char* key);
	bool hasKey(std::string key);


	void setValue(char* key, char* value) { config[key] = value; };

	void printAll(); // for debugging

  //protected:
	std::map<std::string, std::string> config;
};

} // namespace DAVE

#endif // DAVECONFIG_HEADER_INCLUDED
