// generic wrapper for a network library
// link the library of your choice in the corresponding subfolder


#ifndef DAVENETWORK_HEADER_INCLUDED
#define DAVENETWORK_HEADER_INCLUDED

namespace DAVE {

bool isMulticastAddress(const char* address); // automatic recognition of multicast with addresses 224.0.0.0 through 239.255.255.255

// server
class DaveNetworkServerUDP {
  public:
	DaveNetworkServerUDP(bool multicast = false);

	~DaveNetworkServerUDP();

	// address is only used for multicast group
	void addClient(const char* address, int port);
	// WARNING: current implementation may only support a single client / a single multicast address!
	// WARNING: current implementation may only support clients of same type: no mixed multicast and normal addresses possible!

	// status
	void sendStatusToClients(void* data, int size); // unreliable UDP broadcast datagram, low latency, packets may get lost, arrive twice or out of order
	//int receiveMessageFromClient(void* data, int bufferSize, const char* senderAddress);

  private:
	bool multicast;
	void* prdata; // for additional variables
};
/*
class DaveNetworkServerTCP {
  public:
	DaveNetworkServerTCP(const char* address, int port);
	~DaveNetworkServerTCP();

	// events
	void sendEventToClients(void* data, int size); // reliable TCP packets
	//int receiveMessageFromClient(void* data, int bufferSize);
};
*/
// clients
class DaveNetworkClientUDP {
  public:
	DaveNetworkClientUDP(const char* address, int port); // automatic recognition of multicast with addresses 224.0.0.0 through 239.255.255.255
	~DaveNetworkClientUDP();

	int receiveStatusFromServer(void* data, int bufferSize); // returns size of received data (max. bufferSize bytes)
			// data points to memory (... bytes) and is filled with the received bytes
			// if something is received call this several times in a loop to prevent buffer overrun
			// if nothing is received the method may block for a few milliseconds while waiting for a message/time out

	//void sendStatusToServer(void* data, int size) // reliable TCP packets

  private:
	bool multicast;
	void* prdata; // for additional variables
};
/*
class DaveNetworkClientTCP {
  public:
	DaveNetworkClientTCP(const char* address, int port, bool instantConnect = true);
	~DaveNetworkClientTCP();

	//bool connect(float timeout);

	//void sendMessageToServer(void* data, int size) // reliable TCP packets
	int receiveMessageFromServer(void* data, int bufferSize);
};
*/

} // namespace DAVE

#endif // DAVENETWORK_HEADER_INCLUDED
