#ifndef DAVEPROJECTION_HEADER_DEFINED
#define DAVEPROJECTION_HEADER_DEFINED

namespace DAVE {

class DaveProjection {
  public:
	// specify the 4 corners of rectangular image area in world coordinate system
    // all points must lie in a plane and describe a rectangle
	DaveProjection(double lowerLeftX, double lowerLeftY, double lowerLeftZ,
				double lowerRightX, double lowerRightY, double lowerRightZ,
				double upperRightX, double upperRightY, double upperRightZ,
				double upperLeftX, double upperLeftY, double upperLeftZ);
	DaveProjection(float hmdHFov, float hmdAspectRatio); // for HMD use this constructor
	~DaveProjection();

	bool loadCalibMatrix(const char* fileName);

	// set OpenGL projection matrix
	//void setProjectionMatrix(float eyeX, float eyeY, float eyeZ);
	void setProjectionMatrix(float eyeX, float eyeY, float eyeZ, float* head); // head matrix used for rotation for hmd

	// clipping plane distances; default values usually work fine
	double zNear;
	double zFar;

  protected:
	/**
	* calibMatrix holds the calibration matrix for fine tuning projection
	* matrix to physical setup.
	* the matrix is in OpenGL format (column-major order)
	*/
	double* calibMatrix;

	// 4 corners of rectangular image area in world coordinate system
	// all points must lie in a plane and describe a rectangle
	double worldCS_lowerLeftCorner[3];
	double worldCS_lowerRightCorner[3];
	double worldCS_upperRightCorner[3];
	double worldCS_upperLeftCorner[3];

	double worldCS_unitRight[3];
	double worldCS_unitUp[3];
	double worldCS_unitNormal[3];

	// for HMD
	float hmdHFov; // horizontal field of view, by default set to -1; if positive projection for HMD will be used
	float hmdAspectRatio; // width/height
};

} // namespace DAVE

#endif // DAVEPROJECTION_HEADER_DEFINED
