#include "../DaveTracking.h"
#include <iostream>

#include "UDPClient.h"
#include "Marker3DList.h"
#include "RecognizedTargetList.h"

#include <sstream>
#include <ios>

const bool INCLUDE_PACKETNUMBER = true;
UDPClient trClient;

//#define TRACKING_DEBUG_MESSAGES

DAVE::DaveTracking::DaveTracking(const char* address, int port) {
	if (!trClient.setup((char*) address, port)) {
		std::cout << "Tracking: Couldn't start the client with the given network address " << address << ":" << port << std::endl;
	}

/* initialization will be done externally
	eyePosLeft[0] = -0.031f;
	eyePosLeft[1] = 0;
	eyePosLeft[2] = 0;
	eyePosRight[0] = 0.031f;
	eyePosRight[1] = 0;
	eyePosRight[2] = 0;
*/
	// set identity matrices
	for (int i=0; i<4; i++) for (int j=0; j<4; j++) {
		head[4*i+j] = (i == j);
		joystick[4*i+j] = (i == j);
	}
}

DAVE::DaveTracking::~DaveTracking() {
	trClient.close();
}

void setMatrix(CGVOpticalTracking::RecognizedTarget* rtl, float m[16]) {
	// set identity matrix
	for (int i=0; i<4; i++) for (int j=0; j<4; j++) {
		m[4*i+j] = (i == j);
	}
	
	// rotation sub-matrix (3x3)
	CGVOpticalTracking::Vec3 aX = rtl->quatRotation.rotate(CGVOpticalTracking::Vec3(1, 0, 0));
	m[0] = aX.x;
	m[1] = aX.y;
	m[2] = aX.z;

	CGVOpticalTracking::Vec3 aY = rtl->quatRotation.rotate(CGVOpticalTracking::Vec3(0, 1, 0));
	m[4] = aY.x;
	m[5] = aY.y;
	m[6] = aY.z;

	CGVOpticalTracking::Vec3 aZ = rtl->quatRotation.rotate(CGVOpticalTracking::Vec3(0, 0, 1));
	m[8] = aZ.x;
	m[9] = aZ.y;
	m[10] = aZ.z;

	// translation components
	m[12] = rtl->position.x;
	m[13] = rtl->position.y;
	m[14] = rtl->position.z;
}

void DAVE::DaveTracking::update() {
	#ifdef TRACKING_DEBUG_MESSAGES
	std::cout << "Reading Tracking data\n";
	#endif // TRACKING_DEBUG_MESSAGES
	bool ret;
	bool oldDataAvailable;

	ret = trClient.requestData(All,INCLUDE_PACKETNUMBER);
	if (!ret) {
		std::cout << "request Data failed"<<std::endl;
		return;
	}
	ret = trClient.receiveData(oldDataAvailable);
	if (!ret&&!oldDataAvailable) {
		std::cout << "receive Data failed"<<std::endl;
		return;
	} else if(oldDataAvailable) {
		//std::cout << "Timeout getting new data, but old Data available"<<std::endl;
	}

	std::string data = trClient.getData();
	std::string what;
	std::istringstream sstr(data);
	CGVOpticalTracking::Marker3DList vMarker;
	CGVOpticalTracking::RecognizedTargetList vTarget;
	int len;
	sstr >> len;
	#ifdef TRACKING_DEBUG_MESSAGES
	std::cout << "Length: "<<len<<std::endl;
	#endif // TRACKING_DEBUG_MESSAGES
/*
while (sstr) {
char c;
sstr >> c;
std::cout << c;
}
std::cout << std::endl;
*/
    while(sstr) {
      sstr >> what;
	#ifdef TRACKING_DEBUG_MESSAGES
      std::cout << "What: "<<what<<std::endl;
	#endif // TRACKING_DEBUG_MESSAGES
      if(what=="MarkerPos") {
	#ifdef TRACKING_DEBUG_MESSAGES
	std::cout << "MarkerPos"<<std::endl;
	#endif // TRACKING_DEBUG_MESSAGES
        sstr >> vMarker;
      } else if(what=="Target") {
	#ifdef TRACKING_DEBUG_MESSAGES
	std::cout << "Target"<<std::endl;
	#endif // TRACKING_DEBUG_MESSAGES
        sstr >> vTarget;
      }
    }

    for (CGVOpticalTracking::RecognizedTargetList::iterator iter = vTarget.begin(); iter!=vTarget.end(); ++iter) {
	#ifdef TRACKING_DEBUG_MESSAGES
      std::cout << "read recognized target: " << iter->name << std::endl;
      std::cout << "  position: (" << iter->position.x << ", " << iter->position.y << ", " << iter->position.z << ')';
      std::cout << "  quaternion: (" << iter->quatRotation.v_s << ", " << iter->quatRotation.v_v.x << ", " << iter->quatRotation.v_v.y << ", " << iter->quatRotation.v_v.z << ')' << std::endl;
	#endif // TRACKING_DEBUG_MESSAGES

		if (iter->name == "glasses") {
			CGVOpticalTracking::Vec3 eDirLeft = iter->quatRotation.rotate(CGVOpticalTracking::Vec3(-0.031f, -0.01f, -0.02f));
			CGVOpticalTracking::Vec3 eDirRight = iter->quatRotation.rotate(CGVOpticalTracking::Vec3(+0.031f, -0.01f, -0.02f));
			eyePosLeft[0] = eDirLeft.x + iter->position.x;
			eyePosLeft[1] = eDirLeft.y + iter->position.y;
			eyePosLeft[2] = eDirLeft.z + iter->position.z;
			eyePosRight[0] = eDirRight.x + iter->position.x;
			eyePosRight[1] = eDirRight.y + iter->position.y;
			eyePosRight[2] = eDirRight.z + iter->position.z;

			setMatrix(&(*iter), head);
		}

		if (iter->name == "gamepad") setMatrix(&(*iter), joystick);
    }
}
