// to facilitate window handling as stated in the config file
// (((to avoid complications, the windows must be on the same graphics card?)))

#ifndef DAVEWINDOWMGR_HEADER_INCLUDED
#define DAVEWINDOWMGR_HEADER_INCLUDED

#include <vector>
#include <string>

namespace DAVE {

class Dave;
class DaveConfig;
class DaveProjection;

class DaveWindowManager {
  public:
	DaveWindowManager(DaveConfig* config);
	~DaveWindowManager();

	struct WindowStruct {
		int posX, posY; // position in windowed mode (i.e. not fullscreen); in multi montior setups this also determines on which monitor the window will appear
		int width, height; // dimensions in windowed mode (i.e. not fullscreen), including borders and title bar
		DaveProjection* projection;
		void* blendMaskImg; // points to a YImage in current implementation
		int systemWindowId; // in glut this is the number returned by glutCreateWindow()
		std::string title;
		bool fullScreenAtStartup;
		bool quadBuffer;
		bool borderLess;

		void* winUserDataPtr; // for free usage in program, it's useful for texture or vertex buffer object names that must be set up for each window individually, is set to NULL at construction time
	};
	std::vector<WindowStruct> windows;

	struct DBufferStruct {
		int windowIndex; // belongs to window in array windows
		int width, height;
		enum CameraStereoOffsetEnum {LEFT_EYE, RIGHT_EYE, DEFAULT} cameraStereoOffset;
		int quadDrawBuffer; // 0 when double buffer, else GL_BACK_LEFT or GL_BACK_RIGHT
	};
	std::vector<DBufferStruct> dBuffers;

  protected:
	double winGetDouble(char* key, double defaultValue, int winNr = 0);
	int winGetInt(char* key, int defaultValue, int winNr = 0);
	std::string winGetString(char* key, std::string defaultValue, int winNr = 0);
	bool winHasKey(char* key, int winNr = 0);

	DAVE::DaveConfig* config;
	std::string iName;
};

} // namespace DAVE

#endif // DAVEWINDOWMGR_HEADER_INCLUDED
