#include <GL/openglean.h>

// StereoLib
//#include "../src/StereoLib/stereo-glut.hpp"

// DAVE
#include "Dave.h"
#include "DaveConfig.h"

#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include <math.h>

#include "DiffTime.h" // for accurate time measurements (milliseconds)
#include "msSleep.h" // for unified sleep (milliseconds)

struct MouseStruct {
	int x, y;
	int oldX, oldY;
	bool buttonPressed[3];
} mouse = {0, 0, 0, 0, false, false, false};

int modifiers = 0;

struct CameraStruct {
	float x, y, z;
	float camAngle1, camAngle2;
} camera = {0, 0, -1, 0, 0};

struct WindowStruct {
	int posX, posY, width, height;
} window = {10, 10, 400, 300};

DiffTime diffTime, totalTime;


// forward declarations

void initialize();
void drawScene(float totalTime, float diffTime);
void reshape(GLsizei width, GLsizei height);

// DAVE
DAVE::Dave dave;
struct UserSyncDataStruct {
	CameraStruct cam;
};


///////////////////////////////////////////////////////////////////////////////


float dts = 0;
void prepare() {
	dts = diffTime.getSeconds();
	diffTime.start();
}

void finish() {
	glutSwapBuffers();
}

void display() {
	prepare(); // comment this line for StereoLib

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// set projection matrix
	dave.setProjectionMatrix();

	glLoadIdentity();
	glTranslatef(camera.x, camera.y, camera.z);
	glColor3f(1,1,1);

	drawScene(totalTime.getSeconds(), dts);

	// glDrawBuffer( left ? GL_BACK_LEFT : GL_BACK_RIGHT );
	// todo: for stereo: repeat everything for other buffer...


	finish(); // comment this line for StereoLib
}

void update() {
	UserSyncDataStruct* usd = (UserSyncDataStruct*) dave.getSyncBufferUserDataPtr();
	if (dave.isServer()) {
		memcpy(&usd->cam, &camera, sizeof(CameraStruct));
	}

	dave.update();

	if (dave.isClient()) {
		memcpy(&camera, &usd->cam, sizeof(CameraStruct));
	}

	msSleep(2);

	glutPostRedisplay();
}

void init() {
	glEnable(GL_DEPTH_TEST);

	// init
	glClearColor (0.0, 0.0, 0.0, 0.0);

	diffTime.start();
	totalTime.start();

	// print usage information
	printf("drag mouse to move or rotate camera\n");
	printf(" - rotate: left button\n");
	printf(" - move in x/y plane: middle button / left button + CTRL key\n");
	printf(" - move in z/zoom: right button / left button + SHIFT key / mouse wheel for zoom only\n");
	printf("press [esc] to exit\n\n");

	camera.z = -1;

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);

	// setup material and lighting
	GLfloat light_pos0[] = {0.0,0.0,5.0,1.0};
	GLfloat col_specular[] = {1.0,1.0,1.0,1.0};
	GLfloat col_ambient[] = {0,0,1,1.0};
	GLfloat col_diffuse[] = {0.4,0.4,1.0,1.0};
	GLfloat light_ambient[] = {0.2,0.2,0.2,1.0};
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, col_specular);
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, col_ambient);
	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, col_diffuse);
	glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 80.0);
	glLightfv(GL_LIGHT0, GL_POSITION, light_pos0);
	glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
	glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 0.1);
	glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, 0.3);
	glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, 0.04);
	glEnable(GL_LIGHT0);

	update();
}

// letters and some other characters
void keyboardFunc(unsigned char key, int x, int y) {
	switch (key) {
		case 27: // escape
			exit(0);
			break;
   }
}

// mouse motion without button pressed
void mouseMove(int x, int y) {
	mouse.oldX = x;
	mouse.oldY = y;
}

// mouse motion with button pressed
void mouseDrag(int x, int y) {
	modifiers = glutGetModifiers();
	mouse.x = x;
	mouse.y = y;

	int dx = (mouse.x - mouse.oldX);
	int dy = (mouse.y - mouse.oldY);

	// rotate
	if (mouse.buttonPressed[GLUT_LEFT_BUTTON] && (modifiers == 0) ) {
		camera.camAngle1 += 0.4 * dy;
		camera.camAngle2 += 0.4 * dx;
	}

	// pan
	if ((mouse.buttonPressed[GLUT_LEFT_BUTTON] && (modifiers == GLUT_ACTIVE_CTRL))
		|| (mouse.buttonPressed[GLUT_MIDDLE_BUTTON])) {
		camera.x += 0.02 * dx;
		camera.z -= 0.02 * dy;
	}

	// zoom
	if ((mouse.buttonPressed[GLUT_LEFT_BUTTON] && (modifiers == GLUT_ACTIVE_SHIFT))
		|| (mouse.buttonPressed[GLUT_RIGHT_BUTTON])) {
		camera.y += 0.02 * dy;
	}

	mouse.oldX = x;
	mouse.oldY = y;
	glutPostRedisplay();
}

void mouseButton(int button, int mstate, int x, int y) {
	modifiers = glutGetModifiers();
	mouse.oldX = x;
	mouse.oldY = y;
	mouse.buttonPressed[button] = (mstate == GLUT_DOWN);
}

void mouseWheel(int wheel, int direction, int x, int y) {
	camera.y += 0.2 * direction;
}

void reshape(GLsizei width, GLsizei height) {
	glViewport(0, 0, (GLsizei)(width), (GLsizei)(height));
	window.width = width;
	window.height = height;
/*
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	const float FOVY = 60; // vertical field of view
	gluPerspective(FOVY, // field of view in degree
		(GLfloat)(width)/(GLfloat)(height), // aspect ratio
		0.01, 300.0); // z near, z far
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
*/
}

void visible(int vis) {
	if (vis == GLUT_VISIBLE) glutIdleFunc(update);
		else glutIdleFunc(NULL);
}

///////////////////////////////////////////////////////////////////////////////

void drawScene(float totalTime, float diffTime) {
	glRotatef(camera.camAngle1, 1, 0, 0);
	glRotatef(camera.camAngle2, 0, 0, 1);

	// render some grid lines
	glColor3f(0.18, 0.18, 0.18);
	glBegin(GL_LINES);

	for (int i=-5; i<5; i++) {
		glVertex3f(i, -5, 0);
		glVertex3f(i, +5, 0);
		glVertex3f(-5, i, 0);
		glVertex3f(+5, i, 0);
	}
	glEnd();

	// axis labels
	glColor3f(0.5, 0.5, 0.5);
	glLineWidth(2.0);
	glBegin(GL_LINES);
		glVertex2f(0.0, 0.0); // x axis
		glVertex2f(1.0, 0.0);
		glVertex2f(1.1, -0.05); // x
		glVertex2f(1.16, 0.05);
		glVertex2f(1.1, 0.05);
		glVertex2f(1.16, -0.05);

		glVertex2f(0.0, 0.0); // y axis
		glVertex2f(0.0, 1.0);
		glVertex2f(-0.03, 1.1); // y
		glVertex2f(0.03, 1.2);
		glVertex2f(-0.03, 1.2);
		glVertex2f(0.0, 1.15);

		glVertex2f(0.0, 0.0); // z axis
		glVertex3f(0.0, 0.0, 1.0);
		glVertex3f(-0.03, 0.0 ,1.16); // z
		glVertex3f(0.03, 0.0, 1.16);
		glVertex3f(-0.03, 0.0 ,1.1); // z
		glVertex3f(0.03, 0.0, 1.16);
		glVertex3f(-0.03, 0.0 ,1.1); // z
		glVertex3f(0.03, 0.0, 1.1);
	glEnd();
	glLineWidth(1.0);

	glEnable(GL_LIGHTING);

	for (int i=0; i<15; i++) {
		float fi = i/15.0;
		for (int j=-3; j<=3; j++) {
			float r = cos(70*fi) + 2;
			float pos_x = r*cos(fi*6.282);
			float pos_y = r*sin(fi*6.282);
			float pos_z = 0.5*j;

			float col_r = cos(fi*6.282) + 0.4*abs(j);
			float col_g = cos(fi*6.282 + 2.1) + 0.4*abs(j);
			float col_b = cos(fi*6.282 + 4.2) + 0.4*abs(j);

			GLfloat fdiffuse[] = {col_r + 0.3, col_g + 0.3, col_b + 0.3, 1.0};
			glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, fdiffuse);
			GLfloat fambient[] = {col_r, col_g, col_b, 1};
			glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, fambient);

			glPushMatrix();
			glTranslatef(pos_x, pos_y, pos_z);

			glutSolidSphere(0.2, 6, 5);

			glPopMatrix();
		}
	}

	glDisable(GL_LIGHTING);

	// render semi transparent floor
	glColor4f(0.5, 0.5, 0.5, 0.7);
	glBegin(GL_QUADS);
		glVertex3f(-10, -10, -0.01);
		glVertex3f(-10,  10, -0.01);
		glVertex3f( 10,  10, -0.01);
		glVertex3f( 10, -10, -0.01);
	glEnd();
}

///////////////////////////////////////////////////////////////////////////////

int main(int argc, char* argv[]) {
	// DAVE
	dave.parseCmdLine(argc, argv);
	dave.init(sizeof(UserSyncDataStruct));

	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH); // GLUT_STEREO

	// DAVE
	std::string iName = dave.config->getString("InstanceName");
	window.posX = dave.config->getInt(iName + ".windowPosX", window.posX);
	window.posY = dave.config->getInt(iName + ".windowPosY", window.posY);
	window.width = dave.config->getInt(iName + ".windowWidth", window.width);
	window.height = dave.config->getInt(iName + ".windowHeight", window.height);

	glutInitWindowSize(window.width, window.height);
	glutInitWindowPosition(window.posX, window.posY);
	glutCreateWindow("OpenGL example");

	glutKeyboardFunc(keyboardFunc);
	//glutSpecialFunc(keyboardSpecialFunc);
	glutReshapeFunc(reshape);
	glutMotionFunc(mouseDrag);
	glutPassiveMotionFunc(mouseMove);
	glutMouseFunc(mouseButton);
	glutMouseWheelFunc(mouseWheel);
	glutIdleFunc(update);
	glutVisibilityFunc(visible);

	init();
	if (dave.config->getString(iName + ".windowMode") == "FULLSCREEN") {
		glutFullScreen();
		glutSetCursor(GLUT_CURSOR_NONE);
	}

	glutDisplayFunc(display);
	// StereoLib
//	glutStereoPrepareFunc(prepare);
//	glutStereoDisplayFunc(display);
//	glutStereoFinishFunc(finish);

	glutMainLoop();
	return 0;   /* ANSI C requires main to return int. */
}
